%
%                       Robpole Version 1.4
%
%            Yaguang Yang,* Andre L. Tits* and Vasile Sima+
%
%   *Institute for Systems Research and Department of Electrical Engineering
%                      University of Maryland
%                      College Park, MD 20742
%                             USA
%
%   +National Institute for Research & Development in Informatics
%                      Bucharest, Romania
%
%                          January 2003
%
%  Copyright (c) 1996-2003 by Y. Yang, A.L. Tits and V. Sima.
%  All rights reserved.
%
% This function updating complex (j,j+1) columns.
%
% Synopsis:
%            [Q,R,Y]=complex_pair(Q,R,Y,S,n,m,j)
%
% IMPORTANT: j>r must hold.
% ------------------------------
%
% Input arguments:
%   Y: an nxn matrix whose columns have the direction and orientation of
%      the real and imaginary parts of the columns of X, but their length
%      (norm) is arbitrary.
%   Q,R: real matrices of qr decomposition of Y.
%   S: a matrix of nxnm; each block of nxm is the basis of a subspace where
%      an eigenvector is located.
%   n: the dimension of the system matrix (real).
%   m: the column dimension of the input matrix (real).
%   (j,j+1): the pair of COMPLEX eigenvectors to be updated.
%
% Output arguments:
%   Y,Q,R: same meaning as above, after updating columns j and j+1 of Y.
%
% Reference:
% [TY] A. L. Tits and Y. Yang, Globally Convergent Algorithms
% for Robust Pole Assignment by State Feedback, to appear in IEEE
% Trans. on Automatic Control, October 1996.
%
%
%                   Conditions for External Use
%                   ===========================
%
%   1. This code  may not be  distributed to third parties. Interested  
%      parties  should contact  the authors directly.
%   2. If modifications are performed on the code, these modifications 
%      shall be  communicated to  the authors.  The modified  routines 
%      will  remain the sole  property of the authors.
%   3. Due  acknowledgment  must  be  made of the  use of this code in 
%      research  reports  or  publications.  Whenever such reports are 
%      released for  public  access, a copy should be forwarded to the 
%      authors.
%   4. This code may only be used for research and development, unless 
%      it has been agreed  otherwise with the authors in writing.
%
%

function [Q,R,Y]=complex_pair(Q,R,Y,S,n,m,j)
sqrt2=sqrt(2); tolupd=10^-10;

% Get the basis of the appropriate subspace
         Sj=S(:,m*(j-1)+1:m*j);

% Form the old xj, xj^R, and xj^I
         yr=Y(:,j); yi=Y(:,j+1);
         xj=yr+i*yi;

% Step (1) in Section VI in [TY, p 1441]. Rank two qr decomposition
         if j<n-1
           [Q,R]=qrdelete(Q,R,j+1);          
           [Q,R]=qrdelete(Q,R,j);
         else     % Note for complex pair, we take real and imaginary parts.
           R=R(:,1:n-2);
         end
         
% Select u
         u=(Q(:,n-1)+i*Q(:,n))/sqrt2;
         k=length(u); tolerance=2*k*eps;

% Step (2) in Section VI in [TY, p 1441]
         a1=Sj'*u; a2=Sj'*conj(u);

% Step (3) in Section VI in [TY, p 1441] and see Proposition B.2
% Step 1 in Proposition B.2
         norma1=norm(a1); norma2=norm(a2);
% 11/5/96 make the following change
         rank2=1; denom=1;
         a2a1=a2'*a1;
         if abs(abs(a2a1)-norma1*norma2)<tolerance
            absd11=1; absd22=0; dgap=1;
            if norma1<=norma2
               if norma2==0
                  absd11=0; mu1=[1; zeros(max(k-2,0),1)]; rank2=0; dgap=0;
               else
                  mu1=a2/norma2;
                  r=triu(qr([a2 a1]));
                  if abs(r(2,2))<tolerance, rank2=0; end
               end
            else
               mu1=a1/norma1;
               r=triu(qr([a1 a2]));
               if abs(r(2,2))<tolerance, rank2=0; end              
            end
         end
         if rank2==1
% 11/5/96 make the above change
% Step 2 in Proposition B.2
            if a2a1~=0
               absa2a1=abs(a2a1);
               phi=absa2a1/(norma1*norma2);
               gamma=a2a1/absa2a1;
            else
               phi=0; gamma=1;
            end
% Step 3 in Proposition B.2
            sqrt1pphi=sqrt(1+phi); sqrt1mphi=sqrt(1-phi);
            M1=[sqrt1pphi gamma'*sqrt1pphi; sqrt1mphi -gamma'*sqrt1mphi];
            Mhat=M1*[norma1*norma1 0; 0 -norma2*norma2]*M1'/2;
            [U,D]=eig(Mhat);
            absd11=abs(real(D(1,1))); absd22=abs(real(D(2,2)));
% Step 4 in Proposition B.2
            coef1=1/sqrt2/norma1; coef2=gamma/sqrt2/norma2;
            co1=U(1,1)/sqrt1pphi; co3=U(1,2)/sqrt1pphi;
            if phi~=1, 
               co2=U(2,1)/sqrt1mphi; co4=U(2,2)/sqrt1mphi;
            else
               co2=0; co4=0;
            end
            dgap=abs(absd22-absd11);
            if max(absd11,absd22)~=0, denom=max(absd11,absd22); end
            if dgap/denom<tolerance
               mu1=coef1*(co1+co2)*a1+coef2*(co1-co2)*a2;
               mu2=coef1*(co3+co4)*a1+coef2*(co3-co4)*a2;
               if norm(mu2)~=0, mu2=mu2/norm(mu2); end               
            elseif absd22<absd11
               mu1=coef1*(co1+co2)*a1+coef2*(co1-co2)*a2;
            else
               mu1=coef1*(co3+co4)*a1+coef2*(co3-co4)*a2;
            end
            mu1=mu1/norm(mu1);
% 11/5/96 make the following change
         end
% 11/5/96 make the above change

% Step (4) in Section VI in [TY, p 1441]
         if dgap/denom<tolerance
             T=Sj*[mu1 mu2]; xj_new=T*(T'*xj);
         else
            t=Sj*mu1; xj_new=t*(t'*xj);
         end
         nrm=norm(xj_new); if nrm~=0, xj_new=xj_new/nrm; end
         
         % the following 10 lines are added for numerical stability
         if dgap<tolupd
            M=[a1 a2]*[a1 -a2]';
            t_old=Sj'*xj; t_new=Sj'*xj_new;
            det_old=t_old'*M*t_old;
            det_new=t_new'*M*t_new;
            err1=abs(det_old)-abs(det_new);
            if abs(err1)<tolupd & abs(det_new)>0.01
               xj_new=xj;
            end
         end
         
         yr_new=real(xj_new); yi_new=imag(xj_new);
% Change orientation of new column pair if necessary so it is close to
% the old column pair (for stopping criterion).
         if (yr_new'*yr+yi_new'*yi<0) yr_new=-yr_new; yi_new=-yi_new; end
         [Q,R]=qrinsert(Q,R,j,yr_new);
         [Q,R]=qrinsert(Q,R,j+1,yi_new);
         Y=[Y(:,1:j-1) yr_new yi_new Y(:,j+2:n)];
