%
%                       Robpole Version 1.4
%
%            Yaguang Yang,* Andre L. Tits* and Vasile Sima+
%
%   *Institute for Systems Research and Department of Electrical Engineering
%                      University of Maryland
%                      College Park, MD 20742
%                             USA
%
%   +National Institute for Research & Development in Informatics
%                      Bucharest, Romania
%
%                          January 2003
%
%  Copyright (c) 1996-2003 by Y. Yang, A.L. Tits and V. Sima.
%  All rights reserved.
%
% This function updates the first (real) column, as in method 0 of [KNV]. 
% It is invoked when exactly one of the prescribed eigenvalues is real.
%
% Synopsis:
%            [Q,R,Y]=one_column(Q,R,Y,S,n,m)
%
%
% Input arguments:
%   Y: an nxn matrix whose columns have the direction and orientation of
%      the real and imaginary parts of the columns of X, but their length
%      (norm) is arbitrary.
%   Q,R: qr decomposition of Y.
%   S: a matrix of dimension nxnm; each block of dimension nxm is the 
%      basis of a subspace where an eigenvector is located.
%   n: the dimension of the system matrix (real).
%   m: the column dimension of the input matrix (real).
%
% Output arguments:
%   Y,Q,R: same meaning as above, after updating the 1st column of Y.
%
% Reference:
% [KNV] J. Kautsky, N.K. Nichols and P. Van Dooren, Robust pole assignment in
% linear state feedback, Int. J. Control, Vol. 41, pp. 1129-1155, 1985.
%
%
%                   Conditions for External Use
%                   ===========================
%
%   1. This code  may not be  distributed to third parties. Interested  
%      parties  should contact  the authors directly.
%   2. If modifications are performed on the code, these modifications 
%      shall be  communicated to  the authors.  The modified  routines 
%      will  remain the sole  property of the authors.
%   3. Due  acknowledgment  must  be  made of the  use of this code in 
%      research  reports  or  publications.  Whenever such reports are 
%      released for  public  access, a copy should be forwarded to the 
%      authors.
%   4. This code may only be used for research and development, unless 
%      it has been agreed  otherwise with the authors in writing.
%

function [Q,R,Y]=one_column(Q,R,Y,S,n,m)

% Extract the basis of the appropriate subspace
S1=S(:,1:m);
y_old=Y(:,1);

% Rank one QR decomposition
[Q,R]=qrdelete(Q,R,1);
u=Q(:,n);

% Update y_old to y_new
y_new=S1*S1'*u;
nrm=norm(y_new); if nrm~=0, y_new=y_new/nrm; end
% Change orientation of the new column if necessary so it is close to
% the old column (for stopping criterion).
if (y_old'*y_new<0) y_new=-y_new; end

% Rank one QR update
[Q,R]=qrinsert(Q,R,1,y_new);
Y=[y_new Y(:,2:n)];
