%
%                       Robpole Version 1.4
%
%            Yaguang Yang,* Andre L. Tits* and Vasile Sima+
%
%   *Institute for Systems Research and Department of Electrical Engineering
%                      University of Maryland
%                      College Park, MD 20742
%                             USA
%
%   +National Institute for Research & Development in Informatics
%                      Bucharest, Romania
%
%                          January 2003
%
%  Copyright (c) 1996-2003 by Y. Yang, A.L. Tits and V. Sima.
%  All rights reserved.
%
% This function updates the real (j,k) columns
%
% Synopsis:
%            [Q,R,Y]=real_pair(Q,R,Y,S,n,m,r,j,k)
%
% IMPORTANT: j<r, k<r must hold.
% ------------------------------
%
% Input arguments:
%   Y: an nxn matrix whose columns have the direction and orientation of
%      the real and imaginary parts of the columns of X, but their length
%      (norm) is arbitrary.
%   Q,R: real matrices of qr decomposition of Y.
%   S: a matrix of dimension nxnm; each block of dimension nxm is the 
%      basis of a subspace where an eigenvector is located.
%   n: the dimension of the system matrix (real).
%   m: the column dimension of the input matrix (real).
%   r: the number of real eigenvalues to be assigned.
%   (j,k): the indexes of eigenvectors to be updated.
%
% Output arguments:
%   Y,Q,R: same meaning as above, after updating columns j and k of Y.
%
% Reference:
% [TY] A. L. Tits and Y. Yang, Globally Convergent Algorithms
% for Robust Pole Assignment by State Feedback, to appear in IEEE
% Trans. on Automatic Control, October 1996.
%
%
%                   Conditions for External Use
%                   ===========================
%
%   1. This code  may not be  distributed to third parties. Interested  
%      parties  should contact  the authors directly.
%   2. If modifications are performed on the code, these modifications 
%      shall be  communicated to  the authors.  The modified  routines 
%      will  remain the sole  property of the authors.
%   3. Due  acknowledgment  must  be  made of the  use of this code in 
%      research  reports  or  publications.  Whenever such reports are 
%      released for  public  access, a copy should be forwarded to the 
%      authors.
%   4. This code may only be used for research and development, unless 
%      it has been agreed  otherwise with the authors in writing.
%

function [Q,R,Y]=real_pair(Q,R,Y,S,n,m,r,j,k)

tolupd=10^-10;

% By removing the column with larger index first, we zero fewer elements.
% The order in inserting column does also effect the operation count.
% Therefore we swap the index if necessary.
       if j>k
         swap=j;
         j=k;
         k=swap;
       end

% Extract the bases of the appropriate subspaces
       Sj=S(:,m*(j-1)+1:m*j);
       Sk=S(:,m*(k-1)+1:m*k);

% Step (1) in Section VI in [TY, p 1441]. Rank two qr decomposition 
       [Q,R]=qrdelete(Q,R,k);
       [Q,R]=qrdelete(Q,R,j);

% Select u and v according to (19)
       u=Q(:,n-1);
       v=Q(:,n);
       tolerance=2*length(u)*eps;

% Step (2) in Section VI in [TY, p 1441]
       a1=Sj'*u; a2=Sj'*v; b1=Sk'*v; b2=Sk'*u;

% Step (3) in Section VI in [TY, p 1441] and see Proposition B.1
% Step a in Proposition B.1
       norma1=norm(a1); norma2=norm(a2); normb1=norm(b1); normb2=norm(b2);
% Step b in Proposition B.1
       sqrt2=sqrt(2);
       if min(norma1,norma2)~=0,
          phi=(a1/norma1)'*(a2/norma2);
          coef1=1/sqrt2/norma1; coef2=1/sqrt2/norma2;
       else
          phi=0; 
          if norma1==0, coef1=0; else, coef1=1/sqrt2/norma1; end
          if norma2==0, coef2=0; else, coef2=1/sqrt2/norma2; end
       end       
       if min(normb1,normb2)~=0,
          psi=(b1/normb1)'*(b2/normb2);
          coef3=1/sqrt2/normb1; coef4=1/sqrt2/normb2;
       else
          psi=0; 
          if normb1==0, coef3=0; else, coef3=1/sqrt2/normb1; end
          if normb2==0, coef4=0; else, coef4=1/sqrt2/normb2; end
       end
% Step c in Proposition B.1
       sqrt1pphi=sqrt(1+phi); sqrt1ppsi=sqrt(1+psi);
       sqrt1mphi=sqrt(1-phi); sqrt1mpsi=sqrt(1-psi);
       M1=[sqrt1pphi sqrt1pphi; sqrt1mphi -sqrt1mphi];
       M2=[norma1*normb1 0; 0 -norma2*normb2];
       M3=[sqrt1ppsi sqrt1ppsi; sqrt1mpsi -sqrt1mpsi];
       Mhat=M1*M2*M3'/2;
       [U,D,V]=svd(real(Mhat));
% Step d in Proposition B.1
       if sqrt1pphi>tolerance
          co1=U(1,1)/sqrt1pphi; co3=U(1,2)/sqrt1pphi;
       else
          co1=0; co3=0;
       end
       if sqrt1mphi>tolerance
          co2=U(2,1)/sqrt1mphi; co4=U(2,2)/sqrt1mphi;
       else
          co2=0; co4=0;
       end
       if sqrt1ppsi>tolerance
          co5=V(1,1)/sqrt1ppsi; co7=V(1,2)/sqrt1ppsi;
       else
          co5=0; co7=0;
       end
       if sqrt1mpsi>tolerance
          co6=V(2,1)/sqrt1mpsi; co8=V(2,2)/sqrt1mpsi;
       else 
          co6=0; co8=0;
       end
       mu1=coef1*(co1+co2)*a1+coef2*(co1-co2)*a2;
       nu1=coef3*(co5+co6)*b1+coef4*(co5-co6)*b2;
       
% Step (4) in Section VI in [TY, p 1441]
       yj=Y(:,j); yk=Y(:,k);
       if 1-D(2,2)/D(1,1)<tolerance
          mu2=coef1*(co3+co4)*a1+coef2*(co3-co4)*a2;
          nu2=coef3*(co7+co8)*b1+coef4*(co7-co8)*b2;
          T=[Sj*mu1 Sj*mu2; Sk*nu1 Sk*nu2];
          y=T*(T'*[yj; yk]); 
          if (norm(y)<tolerance)
             y=T(:,1);
          end
       else
          y=[Sj*mu1; Sk*nu1];
       end
       yj_new=y(1:n,1);
       yk_new=y(n+1:2*n,1);
       yj_new=yj_new/norm(yj_new);
       yk_new=yk_new/norm(yk_new);
       
       % the following 11 lines are only introduced for numerical stability
       if D(1,1)-D(2,2)<tolupd
          M=[a1 a2]*[b1 -b2]';
          tj_new=Sj'*yj_new; tk_new=Sk'*yk_new;
          tj_old=Sj'*yj; tk_old=Sk'*yk;
          det_new=tj_new'*M*tk_new;
          det_old=tj_old'*M*tk_old; err1=det_new-det_old;
          if (abs(err1)<tolupd & abs(det_new)>0.01) 
             yj_new=yj;
             yk_new=yk;
          end
       end
% Change orientation of new columns if necessary so they are close to
% the old columns (for stopping criterion).
       if (yj_new'*yj<0) yj_new=-yj_new; end
       if (yk_new'*yk<0) yk_new=-yk_new; end
       [Q,R]=qrinsert(Q,R,j,yj_new);
       [Q,R]=qrinsert(Q,R,k,yk_new);
       Y=[Y(:,1:j-1) yj_new Y(:,j+1:k-1) yk_new Y(:,k+1:n)];
