%
%                       Robpole Version 1.5.1
%
%            Yaguang Yang,* Andre L. Tits* and Vasile Sima+
%
%   *Institute for Systems Research and Department of Electrical Engineering
%                      University of Maryland
%                      College Park, MD 20742
%                             USA
%
%   +National Institute for Research & Development in Informatics
%                      Bucharest, Romania
%
%                          January 2014
%
%  Copyright (c) 1996-2014 by Y. Yang, A.L. Tits and V. Sima.
%  All rights reserved.
%
% This function performs robust pole assignment. It implements Algorithm 5.1a
% from [TY].  This algorithm is inspired from a popular algorithm due to
% Kautsky, Nichols and Van Dooren [KNV], but is somewhat faster and can
% handle complex prescribed eigenvalues (in complex conjugate pairs).
%
% NOTE: The computational refinements discussed in Appendices B and C
%       of [TY] are NOT fully implemented here.
%
% Synopsis:
%            [F,iter]=robpole(A,B,lambda,robustPoles,initX,userX,toler,numsweep)
% Input arguments:
%   A: system matrix (real) with dimension nxn.
%   B: input matrix (real) with dimension nxm and rank m, with m>1.
%   lambda: vector of dimension n whose entries are the prescribed closed-
%      loop eigenvalues; the set of prescribed eigenvalues, counting
%      multiplicity, must be self-conjugate.
%   robustPoles: user selected parameter for robust solution or
%      a fast solution. robustPoles = 1 for robust solution
%      robustPoles = 0 for fast solution.
%   initX: choice of [0,1,2]. 0= default initial point is selected as [BN].
%      1= randomly selected initial point similar to [BN] but with random
%      weight. 2= user selected FEASIBLE initial point which MUST be provided,
%   userX: user selected initial point which will be verified 
%      to meet U_1^T(A-lambda_iI)x_i=0. eq.(1) of [TY].
%   toler: user selected tolerance for the stopping criterion; default
%      is 0.0000000001 (10^(-10)). 
%   numsweep: user selected maximum number of sweeps; default is 5. 
%   To get more robustness feedback, up to numsweep=100 may be appropriate.
%
% Output arguments:
%   F:  the feedback matrix; the closed-loop matrix is A+BF.
%   iter: the number of iterations to achieve convergence.
%
% References:
% [TY] A. L. Tits and Y. Yang, Globally Convergent Algorithms
% for Robust Pole Assignment by State Feedback, IEEE Trans. on 
% Automatic Control, Vol. 41, No. 10, October 1996, pp. 1432-1452.
% [KNV] J. Kautsky, N.K. Nichols and P. Van Dooren, Robust Pole Assignment in
% Linear State Feedback, Int. J. Control, Vol. 41, pp. 1129-1155, 1985.
% [BN] R. Byers and S.G. Nash, Approaches to Robust Pole Assignment,
% International Journal of Control, Vol. 45, pp. 97-107, 1989.
%
%
%                   Conditions for External Use
%                   ===========================
%
%   1. If modifications are performed on the code, these modifications 
%      shall be  communicated to  the authors.  The modified  routines 
%      will  remain the sole  property of the authors.
%   2. Due  acknowledgment  must  be  made of the  use of this code in 
%      research  reports  or  publications.  Whenever such reports are 
%      released for  public  access, a copy should be forwarded to the 
%      authors.
%   3. This code may only be used for research and development, unless 
%      it has been agreed  otherwise with the authors in writing.
%
%
%           Enhancements in successive versions of ROBPOLE.m
%           ================================================
%
%   Version 1.5.1 (January 2014) Added two options. First, default 
%       initialization and randomized initialization can be selected.
%       Second, user provided initial point and program generated
%       initial point can be chosen.
%       This version might not work without changes on some older versions
%       of MATLAB. For instance, unused output arguments denoted by ~ might
%       be replaced by dummy variables (replace [~ by [dummy and ~] by
%       dummy]).
%   Version 1.5 (October 2006) Test version 1.4.3 is finalized.
%   Version 1.4.3 (February 2006) The data error message is corrected from
%       "Data error: The multiplicity of a prescribed eigenvalue is greater 
%       than the number of inputs" to "Data error: The multiplicity of a 
%       prescribed eigenvalue is greater than the rank of inputs".
%       The program adopts a heuristic method to make sure that 
%       X is full rank if a user chooses to have a quick solution of 
%       pole assignment but NOT robust pole assignment.
%       Closed loop eigenvalues are allowed to be the same as open loop 
%       eigenvalues. If V1t*(A-ordered_lambda(kk)*eye(n))==0, 
%       the eigenspace dimension is n instead of m.
%   Version 1.4.2 (January 2006) The program is now able to handle the
%       the cases where B is not full rank. These cases have been seen
%       in some real world control systems. SVD decomposition, rather than
%       QR factorization is used for obtaining the bases of assignable
%       subspaces. This needs a little more operations (QR is cheaper than
%       SVD), but it saves some operations when feedback matrix F is
%       calculated because the inverse is taken on a diagonal matrix.
%   Version 1.4.1 (December 2005) A false warning is taken out. Initial
%       X is allowed to be singular. Stop criteria check are modified for 
%       to possible initial singular X. Controllability check is moved 
%       to the very end of the program after either the stop criteria 
%       is met or the program fails.
%   Version 1.4 (January 2003) A bug in Case 2, Steps 2.b and 2.d is 
%       fixed (thanks go to Vasile Sima).  Several other comments due 
%       to Vasile Sima are adopted that make the code cleaner. Another
%       bug requiring normalization of Y is also fixed. The fix involves
%       some check for the columns of Y in every sweep, and rescale 
%       the columns ONLY if they are too small or too big. Also add 
%       "toler" as a user selected tolerance for stopping criterion. 
%       Finally, structure is modified to make the code more concise.
%   Version 1.3 (July 1999): A bug in Case 2, Steps 2.b and 2.d is fixed.
%   Version 1.2 (January 1999): Part B (square B matrix) is rewritten 
%       after several bugs are uncovered.
%   Version 1.1 (December 1996): Two bugs (one in complex_pair.m and one 
%       in robpole.m) are corrected following the failures reported 
%       by Dr. Andras Varga.
%   Version 1.0: Novembre 1996.

function [F,iter]=robpole(A,B,lambda,robustPoles,initX,userX,toler,numsweep)
if exist('robustPoles','var') == 0
    robustPoles = 1;
end
if exist('toler','var')==0
    toler = 0.0000000001;
elseif toler<100*eps 
    toler=100*eps;
end
if exist('numsweep','var')==0
    numsweep = 5;
end
%
% *****************************************************************
% A. Check input data and reorder the entries of lambda in such a way 
%    that the first r entries are real prescribed eigenvalues, and
%    the remaining 2c entries are complex prescribed eigenvalues, 
%    grouped in complex conjugate pairs.
% *****************************************************************

% Check whether A is square
[nx,na] = size(A);
if (nx~=na),
   error('Data error: A is not square matrix!');
end

% Check whether the dimensions of A, B are compatible
[n,m] = size(B);
if (nx ~= n || m==0),
   error('Data error: dimensions of A and B matrices are not compatible.')
end

% Check whether the matrix B is full column rank
if (0 == rank(B))
   error('Data error: B matrix is zero rank.')
end

% Check whether the number of prescribed eigenvalues is equal to n
if length(lambda)~=n,
   error('Data error: the number of prescribed eigenvalues is different from the dimension of the state space.');
end

% Determine the number r of real prescribed eigenvalues, the number c of pairs
% of complex prescribed eigenvalues, and reorder them in ordered_lambda in
% such a way that the real eigenvalues come first.
r=0; c2=0; j=1;
ordered_lambda=zeros(n,1);
while (j<=n),
    if imag(lambda(j))==0.0,
        r=r+1;
        ordered_lambda(r)=lambda(j);
    else
        ordered_lambda(n-c2)=lambda(j);
        c2=c2+1;
    end;
    j=j+1;
end;
c=c2/2;
if (c~=floor(c))
   error('Data error: The set of the prescribed eigenvalues is not self-conjugate')
end

% Reorder the complex prescribed eigenvalues such that ordered_lambda(r+2i)=
% conj(ordered_lambda(r+2i-1)), i=1,...,c.
for ii=r+1:2:n-1
   if ordered_lambda(ii+1)~=conj(ordered_lambda(ii))
      for jj=ii+2:n
         if (ordered_lambda(jj)==conj(ordered_lambda(ii)))
            shift=ordered_lambda(ii+1);
            ordered_lambda(ii+1)=ordered_lambda(jj);
            ordered_lambda(jj)=shift;
            break,
         end
      end
      if (ordered_lambda(ii)~=conj(ordered_lambda(ii+1)))
         error('Data error: The set of the prescribed eigenvalues is not self-conjugate')
      end
   end
end

% *****************************************************************
% B. Special case: (number of inputs)==(number of states). In this
%    case, since it has been checked that B is full column rank, the 
%    closed-loop matrix A+BF can be assigned arbitrarily.  We
%    assign it to a (real) block diagonal normal matrix with scalar blocks
%    equal to prescribed real eigenvalues and 2x2 blocks of the form
%    [a b;-b a], where a+-jb are the prescribed complex eigenvalues.
% *****************************************************************

dlambda=diag(ordered_lambda);
if (m==n),
   AplusBF = real(dlambda);
   j = r+1;
   while (j<=n-1),
         AplusBF(j,j+1) = imag(ordered_lambda(j));
         AplusBF(j+1,j) = - imag(ordered_lambda(j));
         j=j+2;
   end;
   F=B\(AplusBF - A);
   iter = 1;
   return;             % escape here!
end

% *******************************************************
% C. Initialization or provide fast solution
% *******************************************************

% Construct orthonormal bases S_i, and store them in 
% matrix S. The notations and computations are as in [TY].
[Q,R,V]=svd(B);
Rdiag=diag(R(1:m,1:m));
m=sum(Rdiag>100*eps*Rdiag(1));                % rank of B
Z=R(1:m,1:m);   % Z and V0t are used at the very end to form F
V0t=Q(:,1:m)';
V1t=Q(:,m+1:n)';

% Check if there are fewer independent inputs than repeated prescribed
% eigenvalues. (If there are fewer independent inputs than repeated 
% prescribed eigenvalues, the closed-loop system matrix (A+BF) cannot
% be diagonalizable; in such a case, the individual sensitivities could
% be infinity.)
sorted_lambda = sort(ordered_lambda); % sort ordered_lambda so that the
for j=1:n-m                           % repeated eigenvalues are adjacent.
   if all(sorted_lambda(j:j+m) == sorted_lambda(j))
      error('Data error: The multiplicity of a prescribed eigenvalue is greater than the rank of inputs.');
   end
end
clear sorted_lambda;
dimS=zeros(1,n);
S=zeros(n,m*n);
rankS=zeros(1,n);
for kk=1:n
   [Q,R]=qr((V1t*(A-ordered_lambda(kk)*eye(n)))');
   if (norm(R)>100*eps)
       S(:,(kk-1)*m+1:(kk-1)*m+m)=Q(:,n-m+1:n);
       if kk==1
           dimS(kk)=m;
       else
           dimS(kk)=dimS(kk-1)+m;
       end
       rankS(kk)=m;
   else
       S(:,(kk-1)*m+1:(kk-1)*m+n)=Q;
       if kk==1
           dimS(kk)=n;
       else
           dimS(kk)=dimS(kk-1)+n;
       end
       rankS(kk)=n;
   end
end
clear ordered_lambda;

% If m=1, there is only unique solution, and the problem is treated
% uniquely.
if (m==1)
    X=S;
    Y=real(X*dlambda/X);
    F=V(:,1:m)*diag(1./diag(Z))*V0t*(Y-A);
    iter=0;
    return;
end

% If robust pole assignment is requested, default initialization follows [TY].
% Select initial (unnormalized) matrix X by the method used in [BN, p.111], 
% i.e., the jth column of X is along the direction (S_{j1}+...+S_{jm}),
% where S_{j1},...,S_{jm} are orthonormal vectors which form the basis S_j.
% If randomized initialization is used, weighter vector is used for the
% purpose. If user provides an initial point, feasibility will be checked
% before it is used.
% If fast solution is requested, the solution will be given here.
if (robustPoles==1)
    % default initialization
    if exist('initX','var') == 0 || ( exist('initX','var') == 1 && initX==0 )
        X=zeros(n,n);
        for jj=1:n
           for kk=1:m
              X(:,jj)=X(:,jj)+S(:,(jj-1)*m+kk);
           end
           X(:,jj)=X(:,jj)/norm(X(:,jj));
        end
    % randomized initialization
    elseif exist('initX','var') == 1 && initX==1
        X=zeros(n,n); weightV=(rand(m,1)-0.5)*2;
        for jj=1:n
           for kk=1:m
              X(:,jj)=X(:,jj)+weightV(kk)*S(:,(jj-1)*m+kk);
           end
           X(:,jj)=X(:,jj)/norm(X(:,jj));
        end
        clear weightV
    % user provided initial point
    elseif exist('initX','var') == 1 && initX==2
        for jj=1:n
            for kk=jj+1:n
                if norm(userX(:,jj)-userX(:,kk),1)==0
                    error('Data error: userX has repeated columns.');
                end
            end
        end
        X=zeros(n,n); 
        for jj=1:n
            for kk=1:n
                tmppp=V1t*(A-lambda(kk)*eye(n))*userX(:,jj);
                if norm(tmppp)<100*eps % toler
                    X(:,kk)=userX(:,jj); break;
                elseif kk==n
                    error('Data error: userX is not feasible.');
                end
            end
        end
    end
else % fast solution
   warning('fast solution occationally fails, use robust pole is suggested in this case')
   Y=zeros(size(A));
   dimY = rank(Y);
   ii=1;
   while (ii<=n && dimY<n)
       [Q,~]=qr(Y);
       Qi = Q(:, dimY+1:n);
       dimQi=rank(Qi);
       if (ii==1)
           Si=S(:,1:dimS(ii));
       else
           Si=S(:,dimS(ii-1)+1:dimS(ii));
       end
       [~, DD, VV]=svd(Si'*Qi);
       jj=1;
       while ( jj<=rankS(ii) && jj<=dimQi)
           if (DD(jj,jj)>100*eps)
               dimQii=jj;
               jj=jj+1;
           else
               break;
           end
       end
       Qii=Si*Si'*Qi*VV(:,1:dimQii);
       vv=zeros(dimQii,1);
       nomimage=zeros(dimQii,1);
       for kk=1:dimQii
           Qii(:,kk)=Qii(:,kk)/norm(Qii(:,kk));
           if (imag(Si)~=0)
               nomimage(kk)=norm(imag(Qii(:,kk)));
           end
       end
       if (imag(Si)~=0)
           [~,idx]=sort(nomimage);
           for kk=1:dimQii
               vv(idx(kk))=vv(idx(kk))+kk;
           end
       end
       for jj=1:n
           if jj<ii
               for kk=1:dimQii
                   tmpcheck = Y(:,jj)'*Qii(:,kk);
                   if (norm(tmpcheck)>1-100*eps)
                       vv(kk)=vv(kk)-n-1;
                   end
               end
           end
           if jj>ii
               if (jj==1)
                   Sj=S(:,1:dimS(jj));
               else
                   Sj=S(:,dimS(jj-1)+1:dimS(jj));
               end
               for kk=1:dimQii
                   tmpcheck = Sj'*Qii(:,kk);
                   if (norm(tmpcheck)<100*eps)
                       vv(kk)=vv(kk)+1;
                   end
               end
           end
       end
       [~, index]=max(vv);
       Y(:,ii)=Qii(:,index);
       if (imag(Si)==0)
           ii=ii+1;
       else
           Y(:,ii+1)=conj(Y(:,ii));
           ii=ii+2;
       end
       dimY = rank(Y);
       if ii-1>dimY
           warning('initilization may fail')
       end
   end
%    dimY
   X=Y;
   Y=real(X*dlambda/X);
   F=V(:,1:m)*diag(1./diag(Z))*V0t*(Y-A);
   iter=0;
   format short;
   errorEig=norm(cmpoles(diag(dlambda),eig(A+B*F))-diag(dlambda));
   if (errorEig>n*1e-2)
       disp([cmpoles(diag(dlambda),eig(A+B*F)) diag(dlambda)])
       error('fast solution fails, please select robustPoles=1');
   end
   return;
end %if robustPoles selection
if m>1 % if m=1, no need to optimize X since F is unique
   % toldet=10^-1*toler;

% The algorithm iterates on a real matrix Y related to the closed-loop
% eigenvector matrix X (see [TY, Appendix B]).  The columns of Y 
% have the direction and orientation of the real and imaginary parts 
% of the columns of X, but their length (norm) is arbitrary.
   Y=zeros(n); Y(:,1:r) = X(:,1:r);
   for j=r+1:2:n
      Y(:,j:j+1)=[real(X(:,j)) imag(X(:,j))];
   end
   Y_old=Y; %Andre (2005) suggested removing det_old = abs(det(X));
   [Q,R]=qr(Y);
   
   iter=0;

% *****************************************************************
% D. Proceeding iteratively, construct an "optimally robust" closed-
%    loop eigenvector matrix X
% *****************************************************************

   for sweep=1:numsweep 
% -----------------------------------------------------------------
% Case 1: r=1. The sequence of updates is obtained as follows
%
% Step 1: Rank one KNV [KNV] update for the real eigenvector
% Step 2: Rank two update for all complex eigenvectors
%    
% -----------------------------------------------------------------
      if r==1     % r=1, KNV iteration is used
         % Step 1
         [Q,R,Y]=one_column(Q,R,Y,S,n,m);
         iter=iter+0.5; % Since only 1 column has been updated
         % Step 2
         for i1=r+1:2:n-1
           [Q,R,Y]=complex_pair(Q,R,Y,S,n,m,i1);
           iter=iter+1;
         end
      end

% -----------------------------------------------------------------
% Case 2: r is even. The sequence of updates is as obtained at the end
% of Section VI in [TY, p 1442].
%
% Step 1:
%   1.a Pair of columns with indices (2p,1) followed by all pairs
%       with indices of the form (2ii,2ii+1) with ii ranging from 1 to
%       p-1
%   1.b all complex conjugate pairs of columns
%   1.c all pairs of columns with indices of the form (2ii-1,2ii) with ii
%       ranging from 1 to p
%   1.d all complex conjugate pairs of columns
% Step 2: For jj=2 to p-1
%   2.a all pairs of columns with indices of the form (ii, ii+jj) (or 
%       (ii,ii+jj-2p)) with ii ranging from 1 to p
%   2.b all complex conjugate pairs of columns
%   2.c all pairs of columns with indices of the form (ii, ii+jj) (or 
%       (ii,ii+jj-2p)) with ii ranging from p+1 to 2p
%   2.d all complex conjugate pairs of columns
% Step 3:
%   3.a All pairs of columns with indices of the form (ii,ii+p) with 
%       ii ranging from 1 to p
%   3.b all complex conjugate pairs of columns
%
% -----------------------------------------------------------------
      if (floor(r/2)==r/2)
         p=r/2;
         if r==0
            for ii=r+1:2:n-1
                [Q,R,Y]=complex_pair(Q,R,Y,S,n,m,ii);
                iter=iter+1;
            end
         else
            % Step 1.a
            [Q,R,Y]=real_pair(Q,R,Y,S,n,m,r,r,1);
            iter=iter+1;
            for ii=2:2:r-2
                jj=ii+1;
                [Q,R,Y]=real_pair(Q,R,Y,S,n,m,r,ii,jj);
                iter=iter+1;
            end
            % Step 1.b
            for ii=r+1:2:n-1
                [Q,R,Y]=complex_pair(Q,R,Y,S,n,m,ii);
                iter=iter+1;
            end
            % Step 1.c
            for ii=1:2:r-1
                [Q,R,Y]=real_pair(Q,R,Y,S,n,m,r,ii,ii+1);
                iter=iter+1;
            end
            % Step 1.d
            for ii=r+1:2:n-1
                [Q,R,Y]=complex_pair(Q,R,Y,S,n,m,ii);
                iter=iter+1;
            end
            % Step 2
            for jj=2:p-1
                for ii=1:r
                % Steps 2.a and 2.c
                    j1=ii+jj;
                    if j1>2*p, j1=j1-2*p; end
                    [Q,R,Y]=real_pair(Q,R,Y,S,n,m,r,ii,j1);
                    iter=iter+1;
                    % Steps 2.b and 2.d
                    if (ii==p || ii==2*p)
                        for j2=r+1:2:n-1
                            [Q,R,Y]=complex_pair(Q,R,Y,S,n,m,j2);
                            iter=iter+1;
                        end
                    end
                end
            end
            %Step 3
            %Step 3.a
            for ii=1:p
                [Q,R,Y]=real_pair(Q,R,Y,S,n,m,r,ii,ii+p);
                iter=iter+1;
            end
            %Step 3.b
            for ii=r+1:2:n-1
                [Q,R,Y]=complex_pair(Q,R,Y,S,n,m,ii);
                iter=iter+1;
            end
         end
      end
% -----------------------------------------------------------------
% Case 3: r>1 is odd. The sequence of updates is obtained as follows.
%
% Step 1:
%   1.a Pair of columns with indices (r,1) followed by all pairs
%       with indices of the form (2ii,2ii+1) with ii ranging from 1 to
%       (r-1)/2
%   1.b all complex conjugate pairs of columns
%   1.c all pairs of columns with indices of the form (2ii-1,2ii) with ii
%       ranging from 1 to (r-1)/2
%   1.d all complex conjugate pairs of columns
% Step 2: For jj=2 to (r-1)/2
%   2.a all pairs of columns with indices of the form (ii, ii+jj) (or
%       (ii,ii+jj-r)) with ii ranging from 1 to (r+1)/2
%   2.b all complex conjugate pairs of columns
%   2.c all pairs of columns with indices of the form (ii, ii+jj) (or
%       (ii,ii+jj-r)) with ii ranging from (r+1)/2 to r
%   2.d all complex conjugate pairs of columns
%
% -----------------------------------------------------------------
      if (r>1 && floor(r/2)~=r/2) 
         % Step 1 
         for ii=0:2:2*(r-1)
            % Steps 1.a and 1.c
            i1=rem(ii,r); if i1==0, i1=r; end
            i2=rem(ii+1,r); if i2==0, i2=r; end
            [Q,R,Y]=real_pair(Q,R,Y,S,n,m,r,i1,i2);
            iter=iter+1;
            % Steps 1.b and 1.d
            if (ii==(r-1) || ii==2*(r-1))
               for i1=r+1:2:n-1
                  [Q,R,Y]=complex_pair(Q,R,Y,S,n,m,i1);
                  iter=iter+1;
               end
            end
         end
         % For loop from j=2 to (r-1)/2
         for jj=2:(r-1)/2
            % Step 2
            for ii=1:r
               % Steps 2.a and 2.c
               j1=rem(ii+jj,r); if j1==0, j1=r; end
               [Q,R,Y]=real_pair(Q,R,Y,S,n,m,r,ii,j1);
               iter=iter+1;
               % Steps 2.b and 2.d
               if (ii==(r+1)/2 || ii==r)
                  for ll=r+1:2:n-1
                     [Q,R,Y]=complex_pair(Q,R,Y,S,n,m,ll);
                     iter=iter+1;
                  end
               end
            end
         end
      end
      X=zeros(n); X(:,1:r)=Y(:,1:r);
      for jj=r+1:2:n
         X(:,jj)=Y(:,jj)+1i*Y(:,jj+1); X(:,jj+1)=conj(X(:,jj));
      end
      conv=norm(Y-Y_old,1);
      if conv<toler, break, end
      %Andre (2005) suggested removing the next 3 lines
      %det_new=abs(det(X));
      %if (det_new==0 && iter>5*c+r*(r-1)/2), break, end
      %if (det_new~=0 && abs(det_new-det_old)/det_new<toldet), break, end
      Y_old=Y; %Andre (2005) suggested removing det_old=det_new;
   end
end % if m>1

% *******************************************************************
% E. Form feedback matrix.
% *******************************************************************

% Check controllability
if ~(rank(X,eps/(100*n))==n)
    error('Data error: (A,B) is likely uncontrollable or nearly uncontrollable')
    % Unable to construct a nonsingular closed-loop eigenvector matrix
end
% Rounding error may make Y be complex. Use "real" to avoid the effect.
Y=real(X*dlambda/X);
F=V(:,1:m)*diag(1./diag(Z))*V0t*(Y-A);

